from unittest import skipIf, skip  # noqa: F401
from unittest.mock import patch, MagicMock  # noqa: F401

from requests.models import HTTPError

from tests import MOCK_CONNECTIONS, TEST_DRAGONFLY_URL


class MockAPIResponse:
    def __init__(self, json_data, status_code, uri="", content=None, headers=None):
        self.json_data = json_data
        self.status_code = status_code
        self.url = TEST_DRAGONFLY_URL + uri
        self.content = content
        self.headers = headers

    def json(self):
        return self.json_data

    @property
    def data(self):
        return self.json()

    def raise_for_status(self):
        http_error_msg = None
        if 400 <= self.status_code < 500:
            http_error_msg = u"%s Client Error: url: %s" % (
                self.status_code,
                self.url,
            )

        elif 500 <= self.status_code < 600:
            http_error_msg = u"%s Server Error: url: %s" % (
                self.status_code,
                self.url,
            )

        if http_error_msg:
            raise HTTPError(http_error_msg, response=self)


def if_mock_connections(*decorators):
    def apply_all(f):
        for d in reversed(decorators):
            f = d(f)
        return f

    return apply_all if MOCK_CONNECTIONS else lambda x: x


generic_200_mock = if_mock_connections(
    patch(
        "requests.Session.request",
        return_value=MockAPIResponse({}, 200),
    )
)

generic_201_mock = if_mock_connections(
    patch(
        "requests.Session.request",
        return_value=MockAPIResponse({}, 201),
    )
)

generic_204_mock = if_mock_connections(
    patch(
        "requests.Session.request",
        return_value=MockAPIResponse({}, 204),
    )
)

generic_400_mock = if_mock_connections(
    patch(
        "requests.Session.request",
        return_value=MockAPIResponse({}, 400),
    )
)



from . import APIResourceBaseTestCase, APIResource

from tests.mock_utils import generic_200_mock


class SampleResourceTestCase(APIResourceBaseTestCase):
    @property
    def resource(self) -> APIResource:
        return self.df.Sample

    @generic_200_mock
    def test__download(self, *args, **kwargs):
        response = self.resource.download(object_id=self.object_id)
        self.assertEqual(200, response.code)



from . import APIResourceBaseTestCase, APIResource

from tests.mock_utils import generic_200_mock


class UserPreferencesResourceTestCase(APIResourceBaseTestCase):
    @property
    def resource(self) -> APIResource:
        return self.df.UserPreferences

    @generic_200_mock
    def test__update(self, *args, **kwargs):
        response = self.resource.update(
            object_id=self.object_id,
            data=self.resource.UpdateUserPreferencesRequestBody(
                apistructure_ignore_list=["x", "y", "z"]
            ),
        )
        self.assertEqual(200, response.code)



# flake8: noqa
from .sdk.dragonfly import Dragonfly
from .sdk.exceptions import DragonflyException
from django_rest_client.types import TParams
from django_rest_client.api_response import APIResponse



import dataclasses
from typing import Dict, List, Optional, Set

from django_rest_client import (
    APIResource,
    APIResponse,
    CreateableAPIResourceMixin,
    ListableAPIResourceMixin,
    PaginationAPIResourceMixin,
    RetrievableAPIResourceMixin,
)
from django_rest_client.types import Toid, TParams
from typing_extensions import Literal

from pydragonfly.sdk.const import ANALYZED, CLEAN, FAILED, REVOKED

from ._utils import omit_keys
from .report import Report


class AnalysisResult:
    """
    .. versionadded:: 0.0.4
    """

    @dataclasses.dataclass
    class RuleResult:
        name: str
        weight: int
        score: int

        def __hash__(self):
            return hash((self.name, self.weight))

        def __eq__(self, other):
            return self.name == other.name and self.weight == other.weight

    # defaults
    id: Toid
    created_at: str
    gui_url: str
    api_url: str
    status: str
    evaluation: str = CLEAN
    weight: int = 0
    malware_families: List[str] = []
    #: deprecated in favor of ``mitre_techniques``.
    malware_behaviours: List[str] = []
    mitre_techniques: List[Dict] = []
    sample: Dict = {}
    reports: List[Dict] = []
    matched_rules: List[RuleResult] = []
    # extras
    score: int = 0
    malware_family: Optional[str] = None
    #: deprecated
    malware_behaviour: Optional[str] = None
    errors: List[str] = []

    def __init__(self, analysis_id: Toid):
        self.id = analysis_id
        # fetch and populate
        self.refresh()

    def __dict__(self) -> dict:
        return {
            "id": self.id,
            "created_at": self.created_at,
            "gui_url": self.gui_url,
            "api_url": self.api_url,
            "status": self.status,
            "evaluation": self.evaluation,
            "weight": self.weight,
            "score": self.score,
            "malware_families": self.malware_families,
            "malware_behaviours": self.malware_behaviours,
            "mitre_techniques": self.mitre_techniques,
            "sample": self.sample,
            "reports": self.reports,
            "matched_rules": [dataclasses.asdict(mr) for mr in self.matched_rules],
        }

    def asdict(self) -> dict:
        return self.__dict__()

    def is_ready(self) -> bool:
        return self.status in [ANALYZED, FAILED, REVOKED]

    def refresh(self) -> None:
        """
        Refetch result from server.
        """
        data = self.__fetch()
        self.__populate(data)

    def __fetch(self) -> dict:
        matched_rules = []
        data = Analysis.retrieve(
            object_id=self.id,
            params=TParams(
                expand=["sample"],
                omit=[
                    "sample.file_deleted",
                    "sample.sections",
                    "sample.flags",
                    "sample.dlls_imported",
                    "sample.entry_points",
                    "sample.user",
                    "sample.file_version_info",
                ],
            ),
        ).data
        self.status = data["status"]
        if self.is_ready():
            for report in data["reports"]:  # we fetch matched-rules against each report
                rules = Report.matched_rules(
                    object_id=report["id"]
                ).data  # this can raise an exception
                if rules:
                    matched_rules.extend(rules)

        return {**data, "matched_rules": matched_rules}

    @staticmethod
    def parse_mitre_techniques(mitre_techniques: List[Dict]) -> List[Dict]:
        return (
            [
                {
                    "tid": tactic["tid"],
                    "name": tactic["name"],
                    "techniques": [
                        {
                            "tid": technique["tid"],
                            "name": technique["name"],
                        }
                        for technique in tactic["techniques"]
                    ],
                }
                for tactic in mitre_techniques
                if tactic["techniques"]
            ]
            if mitre_techniques
            else []
        )

    def __populate(self, data: dict) -> None:
        # defaults
        self.created_at = data["created_at"]
        self.gui_url = data["gui_url"]
        self.api_url = data["api_url"]
        self.status = data["status"]
        self.evaluation = data["evaluation"]
        self.weight = data["weight"]
        self.malware_families = data["malware_families"]
        self.mitre_techniques = self.parse_mitre_techniques(
            data.get("mitre_techniques", [])
        )
        self.malware_behaviours = [  # deprecated
            technique["name"]
            for tactic in self.mitre_techniques
            for technique in tactic["techniques"]
        ]
        self.sample = data["sample"]
        self.reports = [
            omit_keys(report, ["time", "malware_families"])
            for report in data["reports"]
        ]
        matched_rules: Set[AnalysisResult.RuleResult] = set()
        for rule in data["matched_rules"]:
            matched_rules.add(
                AnalysisResult.RuleResult(
                    name=rule["rule"],
                    weight=rule["weight"],
                    score=(
                        round(min(100, rule["weight"]) / 10)
                        if rule["weight"] != 0
                        else 0
                    ),
                )
            )
        self.matched_rules = list(matched_rules)
        # extra
        self.score = round(min(100, self.weight) / 10) if self.weight != 0 else 0
        self.malware_family = (
            self.malware_families[0] if self.malware_families else None
        )
        self.malware_behaviour = (  # deprecated
            self.malware_behaviours[0] if self.malware_behaviours else None
        )
        self.errors = list(
            set([report["error"] for report in self.reports if report["error"]])
        )


@dataclasses.dataclass
class CreateAnalysisRequestBody:
    profiles: List[int]
    private: bool = False
    allow_actions: bool = False
    root: bool = False
    os: Optional[Literal["WINDOWS", "LINUX"]] = None
    arguments: Optional[List[str]] = None
    dll_entrypoints: Optional[List[str]] = None


class Analysis(
    APIResource,
    RetrievableAPIResourceMixin,
    ListableAPIResourceMixin,
    CreateableAPIResourceMixin,
    PaginationAPIResourceMixin,
):
    """
    :class:`pydragonfly.Dragonfly.Analysis`
    """

    OBJECT_NAME = "api.analysis"
    EXPANDABLE_FIELDS = {
        "retrieve": ["sample", "reports"],
        "list": [],
    }
    ORDERING_FIELDS = [
        "created_at",
        "sample__filename",
        "weight",
    ]
    CreateAnalysisRequestBody = CreateAnalysisRequestBody
    Result = AnalysisResult

    @classmethod
    def retrieve(
        cls,
        object_id: Toid,
        params: Optional[TParams] = None,
    ) -> APIResponse:

        result = super().retrieve(object_id=object_id, params=params)

        result.data["mitre_techniques"] = AnalysisResult.parse_mitre_techniques(
            result.data.get("mitre_techniques", [])
        )
        # for backwards compatibility
        result.data["malware_behaviours"] = [  # deprecated
            technique["name"]
            for tactic in result.data["mitre_techniques"]
            for technique in tactic["techniques"]
        ]
        return result

    @classmethod
    def create(
        cls,
        data: CreateAnalysisRequestBody,
        sample_name: str,
        sample_buffer: bytes,
        params: Optional[TParams] = None,
    ) -> APIResponse:
        # first: POST sample uploading it
        resp1 = cls._request(
            "POST",
            url="api/sample",
            files={"sample": (sample_name, sample_buffer)},
        )
        # second: POST analysis using the new sample ID
        # build request body
        req_data = {
            **{k: v for k, v in dataclasses.asdict(data).items() if v is not None},
            "sample_id": resp1.data["id"],
        }
        if resp1.data["malware_type"] == "DLL":
            req_data["dll_entrypoints"] = (
                data.dll_entrypoints
                if data.dll_entrypoints
                else resp1.data["entry_points"]
            )  # dll_entrypoints is required in case of DLL
        resp2 = cls._request(
            "POST",
            url=cls.class_url(),
            json=req_data,
            params=params,
        )
        return resp2

    @classmethod
    def aggregate_evaluations(
        cls,
        params: Optional[TParams] = None,
    ) -> APIResponse:
        url = cls.class_url() + "/aggregate/evaluations"
        return cls._request("GET", url=url, params=params)

    @classmethod
    def aggregate_status(
        cls,
        params: Optional[TParams] = None,
    ) -> APIResponse:
        url = cls.class_url() + "/aggregate/status"
        return cls._request("GET", url=url, params=params)

    @classmethod
    def aggregate_malware_families(
        cls,
        params: Optional[TParams] = None,
    ) -> APIResponse:
        url = cls.class_url() + "/aggregate/malware_families"
        return cls._request("GET", url=url, params=params)

    @classmethod
    def aggregate_malware_type(
        cls,
        params: Optional[TParams] = None,
    ) -> APIResponse:
        url = cls.class_url() + "/aggregate/malware_type"
        return cls._request("GET", url=url, params=params)

    @classmethod
    def revoke(
        cls,
        object_id: Toid,
        params: Optional[TParams] = None,
    ) -> APIResponse:
        url = cls.instance_url(object_id) + "/revoke"
        return cls._request("POST", url=url, params=params)



from abc import abstractmethod
from unittest import TestCase, SkipTest

from django_rest_client import APIResource
from pydragonfly import Dragonfly, DragonflyException

from tests import TEST_DRAGONFLY_API_KEY, TEST_DRAGONFLY_URL
from tests.mock_utils import patch, if_mock_connections, MockAPIResponse


class APIResourceBaseTestCase(TestCase):
    object_id = 1
    data = {}

    @classmethod
    def setUpClass(cls) -> None:
        if cls == APIResourceBaseTestCase:
            raise SkipTest(f"{cls.__name__} is an abstract base class.")
        # else, setup Dragonfly client instance
        cls.df = Dragonfly(api_key=TEST_DRAGONFLY_API_KEY)
        cls.df._server_url = TEST_DRAGONFLY_URL
        return super().setUpClass()

    @property
    @abstractmethod
    def resource(self) -> APIResource:
        raise NotImplementedError()

    @if_mock_connections(
        patch(
            "requests.Session.request",
            side_effect=[
                MockAPIResponse({}, 200),
                MockAPIResponse({}, 400),
            ],
        )
    )
    def test__retrieve(self, *args, **kwargs):
        retrieve_fn = getattr(self.resource, "retrieve", None)
        if retrieve_fn:
            # a) ok
            response = retrieve_fn(object_id=self.object_id)
            self.assertEqual(200, response.code)
            # b) error
            with self.assertRaises(DragonflyException):
                response = retrieve_fn(object_id=self.object_id)
                self.assertEqual(400, response.code)

    @if_mock_connections(
        patch(
            "requests.Session.request",
            side_effect=[
                MockAPIResponse({}, 200),
                MockAPIResponse({}, 400),
            ],
        )
    )
    def test__list(self, *args, **kwargs):
        list_fn = getattr(self.resource, "list", None)
        if list_fn:
            # a) ok
            response = list_fn()
            self.assertEqual(200, response.code)
            # b) error
            with self.assertRaises(DragonflyException):
                response = list_fn()
                self.assertEqual(400, response.code)

    @if_mock_connections(
        patch(
            "requests.Session.request",
            side_effect=[
                MockAPIResponse({}, 201),
                MockAPIResponse({}, 400),
            ],
        )
    )
    def test__create(self, *args, **kwargs):
        create_fn = getattr(self.resource, "create", None)
        if create_fn:
            # a) ok
            response = create_fn(data=self.data)
            self.assertEqual(201, response.code)
            # b) error
            with self.assertRaises(DragonflyException):
                response = create_fn(data=self.data)
                self.assertEqual(400, response.code)

    @if_mock_connections(
        patch(
            "requests.Session.request",
            side_effect=[
                MockAPIResponse({}, 200),
                MockAPIResponse({}, 400),
            ],
        )
    )
    def test__update(self, *args, **kwargs):
        update_fn = getattr(self.resource, "update", None)
        if update_fn:
            # a) ok
            response = update_fn(object_id=self.object_id, data=self.data)
            self.assertEqual(200, response.code)
            # b) error
            with self.assertRaises(DragonflyException):
                response = update_fn(object_id=self.object_id, data=self.data)
                self.assertEqual(400, response.code)

    @if_mock_connections(
        patch(
            "requests.Session.request",
            side_effect=[
                MockAPIResponse({}, 204),
                MockAPIResponse({}, 400),
            ],
        )
    )
    def test__delete(self, *args, **kwargs):
        delete_fn = getattr(self.resource, "delete", None)
        if delete_fn:
            # a) ok
            response = delete_fn(object_id=self.object_id)
            self.assertEqual(204, response.code)
            # b) error
            with self.assertRaises(DragonflyException):
                response = delete_fn(object_id=self.object_id)
                self.assertEqual(400, response.code)

    @if_mock_connections(
        patch(
            "requests.Session.request",
            side_effect=[
                MockAPIResponse({"total_pages": 2}, 200),
                MockAPIResponse({"total_pages": 2}, 200),
                MockAPIResponse({"total_pages": 2}, 400),
                MockAPIResponse({"total_pages": 2}, 400),
            ],
        )
    )
    def test__auto_paging_iter(self, *args, **kwargs):
        page_iter_fn = getattr(self.resource, "auto_paging_iter", None)
        if page_iter_fn:
            # a) ok
            for response, page in page_iter_fn():
                self.assertEqual(200, response.code)
            # b) error
            with self.assertRaises(DragonflyException):
                for response, page in page_iter_fn():
                    self.assertEqual(200, response.code)

    @if_mock_connections(
        patch(
            "requests.Session.request",
            side_effect=[
                MockAPIResponse({}, 200),
                MockAPIResponse({}, 400),
            ],
        )
    )
    def test__get(self, *args, **kwargs):
        get_fn = getattr(self.resource, "get", None)
        if get_fn:
            # a) ok
            response = get_fn()
            self.assertEqual(200, response.code)
            # b) error
            with self.assertRaises(DragonflyException):
                response = get_fn()
                self.assertEqual(400, response.code)



import dataclasses
from typing import List, Optional

from django_rest_client import (
    APIResource,
    APIResponse,
    CreateableAPIResourceMixin,
    ListableAPIResourceMixin,
    PaginationAPIResourceMixin,
    RetrievableAPIResourceMixin,
    UpdateableAPIResourceMixin,
)
from django_rest_client.types import Toid, TParams


@dataclasses.dataclass
class CreateRuleRequestBody:
    rule: str
    weight: int
    modules: dict
    variables: List[str] = dataclasses.field(default_factory=list)
    malware_family: str = ""
    mitre_technique: str = None
    meta_description: dict = dataclasses.field(default_factory=dict)
    sensitive: bool = False


@dataclasses.dataclass
class UpdateRuleRequestBody:
    enabled: bool


class Rule(
    APIResource,
    RetrievableAPIResourceMixin,
    ListableAPIResourceMixin,
    CreateableAPIResourceMixin,
    UpdateableAPIResourceMixin,
    PaginationAPIResourceMixin,
):
    """
    :class:`pydragonfly.Dragonfly.Rule`
    """

    OBJECT_NAME = "api.rule"
    EXPANDABLE_FIELDS = {
        "retrieve": ["user", "actions", "clause", "permissions"],
        "list": ["user", "permissions"],
    }
    ORDERING_FIELDS = [
        "created_at",
        "rule",
        "weight",
        "malware_family",
        "mitre_technique",
    ]

    # models
    CreateRuleRequestBody = CreateRuleRequestBody
    UpdateRuleRequestBody = UpdateRuleRequestBody

    @classmethod
    def create(
        cls,
        data: CreateRuleRequestBody,
        params: Optional[TParams] = None,
    ) -> APIResponse:
        post_data = dataclasses.asdict(data)
        return super().create(data=post_data, params=params)

    @classmethod
    def update(
        cls,
        object_id: Toid,
        data: UpdateRuleRequestBody,
        params: Optional[TParams] = None,
    ) -> APIResponse:
        post_data = {
            "enabled": data.enabled,
        }
        return super().update(object_id=object_id, data=post_data, params=params)

    @classmethod
    def mitre(cls, params: Optional[TParams] = None) -> APIResponse:
        """
        .. versionadded:: 0.1.0
        """
        url = cls.class_url() + "/mitre"
        return cls._request("GET", url=url, params=params)

    @classmethod
    def aggregate_malware_behaviour(
        cls,
        params: Optional[TParams] = None,
    ) -> APIResponse:
        """
        Deprecated in favor of ``aggregate_mitre_technique``.
        Will be removed in next release.
        """
        return cls.aggregate_mitre_technique(params=params)

    @classmethod
    def aggregate_mitre_technique(
        cls,
        params: Optional[TParams] = None,
    ) -> APIResponse:
        """
        .. versionadded:: 0.1.0
        """
        url = cls.class_url() + "/aggregate/mitre_technique"
        return cls._request("GET", url=url, params=params)

    @classmethod
    def aggregate_behaviour(
        cls,
        params: Optional[TParams] = None,
    ) -> APIResponse:
        """
        .. versionadded:: 0.1.0
        """
        url = cls.class_url() + "/aggregate/behaviour"
        return cls._request("GET", url=url, params=params)



import click
from rich import print as rprint

from pydragonfly import DragonflyException, TParams
from .._utils import (
    ClickContext,
    json_flag_option,
    add_options,
)
from ._renderables import _paginate_table, _generate_rule_table


@click.group("rule")
def rule():
    """
    Rule Management\n
    >>> [API] https://dragonfly.certego.net/api/rule\n
    >>> [GUI] https://dragonfly.certego.net/dashboard/rules
    """


@rule.command("list", help="List all rules")
@add_options(json_flag_option)
@click.pass_context
def rule_list(ctx: ClickContext, as_json: bool):
    ctx.obj._logger.info("Requesting list of rules..")
    ctx.obj._logger.info(f"[+] GUI: {ctx.obj._server_url}/dashboard/rules")
    params = TParams(ordering=["-created_at"])
    try:
        if as_json:
            response = ctx.obj.Rule.list(params=params)
            rprint(response.data)
        else:
            generator = ctx.obj.Rule.auto_paging_iter(params=params)
            _paginate_table(generator, _generate_rule_table)
    except DragonflyException as exc:
        ctx.obj._logger.fatal(str(exc))


@rule.command("retrieve", help="Retrieve rule object")
@click.argument("rule_id", type=int)
@click.pass_context
def rule_retrieve(ctx: ClickContext, rule_id: int):
    ctx.obj._logger.info(f"Requesting rule [underline blue]#{rule_id}[/]..")
    try:
        response = ctx.obj.Rule.retrieve(
            object_id=rule_id, params=TParams(expand=["user"])
        )
        rprint(response.data)
    except DragonflyException as exc:
        ctx.obj._logger.fatal(str(exc))



from django_rest_client import (
    APIResource,
    RetrievableAPIResourceMixin,
    ListableAPIResourceMixin,
    PaginationAPIResourceMixin,
)


class Action(
    APIResource,
    RetrievableAPIResourceMixin,
    ListableAPIResourceMixin,
    PaginationAPIResourceMixin,
):
    """
    :class:`pydragonfly.Dragonfly.Action`
    """

    OBJECT_NAME = "api.action"
    EXPANDABLE_FIELDS = {
        "retrieve": ["user"],
        "list": ["user"],
    }
    ORDERING_FIELDS = []



from unittest import TestCase

from pydragonfly.sdk.const import ANALYZED, MALICIOUS
from pydragonfly.sdk.resources.analysis import Analysis
from tests.mock_utils import (
    MockAPIResponse,
    generic_200_mock,
    generic_201_mock,
    generic_204_mock,
    if_mock_connections,
    patch,
)

from . import APIResource, APIResourceBaseTestCase


class AnalysisResultTestCase(TestCase):

    matched_rules_json = [
        {
            "rule": "TestRule",
            "weight": 97,
        }
    ]

    result_json = {
        "id": 12,
        "created_at": "2022-01-17T12:07:55.446274Z",
        "status": "ANALYZED",
        "evaluation": "MALICIOUS",
        "weight": 120,
        "malware_families": ["Ransomware", "Trojan"],
        "mitre_techniques": [
            {
                "tid": "tactic_tid",
                "name": "test_tactic",
                "techniques": [
                    {
                        "tid": "technique_tid",
                        "name": "test_technique",
                    }
                ],
            }
        ],
        "sample": {"id": 1, "filename": "test"},
        "reports": [
            {
                "id": 1,
                "error": "Internal error",
                "profile": {"id": 1, "filename": "test.ql"},
            },
        ],
        "gui_url": "https://dragonfly.certego.net/analysis/12",
        "api_url": "https://dragonfly.certego.net/api/analysis/12",
    }

    @patch(
        "pydragonfly.sdk.resources.analysis.Analysis.retrieve",
        return_value=MockAPIResponse(result_json, 200),
    )
    @patch(
        "pydragonfly.sdk.resources.report.Report.matched_rules",
        return_value=MockAPIResponse(matched_rules_json, 200),
    )
    def test_populate(self, *args, **kwargs):
        result = Analysis.Result(12)
        self.assertTrue(result.is_ready())
        self.assertEqual(result.id, 12)
        self.assertEqual(result.status, ANALYZED)
        self.assertEqual(result.evaluation, MALICIOUS)
        self.assertEqual(result.reports, self.result_json["reports"])
        self.assertEqual(result.score, 10)
        self.assertEqual(result.malware_family, "Ransomware")
        self.assertEqual(result.malware_behaviour, "test_technique")
        self.assertEqual(result.malware_behaviours, ["test_technique"])
        self.assertEqual(result.mitre_techniques, self.result_json["mitre_techniques"])
        self.assertEqual(result.matched_rules[0].name, "TestRule")
        self.assertEqual(result.matched_rules[0].weight, 97)
        self.assertEqual(result.matched_rules[0].score, 10)
        self.assertDictEqual(
            result.asdict(),
            {
                **self.result_json,
                "score": 10,
                "matched_rules": [
                    {
                        "name": self.matched_rules_json[0]["rule"],
                        "weight": self.matched_rules_json[0]["weight"],
                        "score": 10,
                    }
                ],
                "malware_behaviours": ["test_technique"],
            },
        )


class AnalysisResourceTestCase(APIResourceBaseTestCase):
    @property
    def resource(self) -> APIResource:
        return self.df.Analysis

    @if_mock_connections(
        patch(
            "requests.Session.request",
            return_value=MockAPIResponse(
                {"id": 1, "malware_type": "DLL", "entry_points": []}, 201
            ),
        )
    )  # POST /api/sample
    @generic_201_mock  # POST /api/analysis
    def test__create(self, *args, **kwargs):
        response = self.resource.create(
            data=self.resource.CreateAnalysisRequestBody(profiles=[1]),
            sample_name="test.exe",
            sample_buffer=b"",
        )
        self.assertEqual(201, response.code)

    @generic_200_mock
    def test__aggregate_evaluations(self, *args, **kwargs):
        response = self.resource.aggregate_evaluations()
        self.assertEqual(200, response.code)

    @generic_200_mock
    def test__aggregate_status(self, *args, **kwargs):
        response = self.resource.aggregate_status()
        self.assertEqual(200, response.code)

    @generic_200_mock
    def test__aggregate_malware_families(self, *args, **kwargs):
        response = self.resource.aggregate_malware_families()
        self.assertEqual(200, response.code)

    @generic_200_mock
    def test__aggregate_malware_type(self, *args, **kwargs):
        response = self.resource.aggregate_malware_type()
        self.assertEqual(200, response.code)

    @generic_204_mock
    def test__revoke(self, *args, **kwargs):
        response = self.resource.revoke(object_id=self.object_id)
        self.assertEqual(204, response.code)



from typing import Optional

from django_rest_client import (
    APIResource,
    APIResponse,
    ListableAPIResourceMixin,
    DeletableAPIResourceMixin,
)
from django_rest_client.types import Toid, TParams


class Invitation(
    APIResource,
    ListableAPIResourceMixin,
    DeletableAPIResourceMixin,
):
    """
    :class:`pydragonfly.Dragonfly.Invitation`
    """

    OBJECT_NAME = "api.me.invitations"
    EXPANDABLE_FIELDS = {
        "retrieve": [],
        "list": [],
    }
    ORDERING_FIELDS = []

    @classmethod
    def accept(
        cls,
        object_id: Toid,
        params: Optional[TParams] = None,
    ) -> APIResponse:
        url = cls.instance_url(object_id) + "/accept"
        return cls._request("POST", url=url, params=params)

    @classmethod
    def decline(
        cls,
        object_id: Toid,
        params: Optional[TParams] = None,
    ) -> APIResponse:
        url = cls.instance_url(object_id) + "/decline"
        return cls._request("POST", url=url, params=params)



# flake8: noqa: E501
from typing import Dict, List

from rich import box
from rich.console import Console, RenderGroup
from rich.emoji import Emoji
from rich.panel import Panel
from rich.table import Table

from .._utils import (
    get_datetime_text,
    get_json_syntax,
    get_status_text,
    get_success_text,
    get_weight_text,
)


def _display_single_analysis(data: Dict) -> None:
    console = Console()
    style = "[bold #31DDCF]"
    user = data["user"]
    sample = data["sample"]
    panels = [
        Panel(
            RenderGroup(
                f"[i green]{Emoji('information')} Visit the GUI page to see all info[/]",
            ),
        ),
        Panel(
            RenderGroup(
                f"{style}ID:[/] {data['id']}",
                f"{style}Date:[/] {data['created_at']}",
                f"{style}User:[/] {user['username']}",
                f"{style}Public:[/] {data['public']}",
                f"{style}Root:[/] {data['root']}",
                f"{style}URL:[/] [u cyan]{data['gui_url']}[/]",
            ),
            title="Attributes",
        ),
        Panel(
            RenderGroup(
                f"{style}Name:[/] {sample['filename']}",
                f"{style}Behaviour:[/] {sample['malware_type']} ({sample['mimetype']})",
                f"{style}Operating System:[/] {sample['os']} ({sample['arch']}, {sample['mode']})",
                f"{style}SHA256:[/] {sample['sha256']}",
                f"{style}SHA1:[/] {sample['sha1']}",
                f"{style}MD5:[/] {sample['md5']}",
            ),
            title="Sample Information",
        ),
        Panel(
            RenderGroup(
                f"{style}Status:[/] {get_status_text(data['status'], False)}",
                f"{style}Evaluation:[/] {get_status_text(data['evaluation'], False)}",
                f"{style}Weight:[/] {data['weight']}",
                f"{style}Malware Families:[/] " + ",".join(data["malware_families"]),
                f"{style}Mitre Techniques:[/] "
                + ",".join(
                    f"[link={technique['mitre_url']}]{technique['name']}[/link]"
                    for tactic in data["mitre_techniques"]
                    for technique in tactic["techniques"]
                ),
            ),
            title="Result Overview",
        ),
        Panel(
            RenderGroup(
                *[
                    RenderGroup(
                        f"{style}Report[/] - [cyan]#{report['id']} \[{report['profile']['filename']}, {report['profile']['emulator']}][/]",
                        f"\tStatus: {get_status_text(report['status'], False)}",
                        f"\tEvaluation: {get_status_text(report['evaluation'], False)}",
                        f"\tWeight: {report['weight']}",
                    )
                    for report in data["reports"]
                ]
            ),
            title="Reports",
        ),
        Panel(get_json_syntax(data), title="Raw"),
    ]
    with console.pager(styles=True):
        for p in panels:
            console.print(p)


def _generate_analysis_table(rows: List[Dict]) -> Table:
    table = Table(
        show_header=True, title="Latest Analysis", box=box.DOUBLE_EDGE, expand=True
    )
    for col in [
        "ID",
        "Created",
        "Sample",
        "Mitre\nTechniques",
        "Malware\nFamilies",
        "Status",
        "Evaluation",
        "Weight",
    ]:
        table.add_column(header=col, header_style="bold blue")
    for el in rows:
        table.add_row(
            f"[link={el['gui_url']}]{Emoji('link')} {el['id']}[/link]",
            get_datetime_text(el["created_at"]),
            f"{el['sample']['filename']}\n({el['sample']['os']}, {el['sample']['arch']}, {el['sample']['mode']})",
            ",".join(
                f"[link={tactic['mitre_url']}]{tactic['name']}[/link]"
                for tactic in el["mitre_techniques"]
            ),
            ",".join(el["malware_families"]),
            get_status_text(el["status"]),
            get_status_text(el["evaluation"]),
            get_weight_text(el["weight"]),
        )
    return table


def _generate_profile_table(rows: List[Dict]) -> Table:
    table = Table(
        show_header=True, title="Latest Profiles", box=box.DOUBLE_EDGE, expand=True
    )
    for col in ["ID", "Enabled", "Created", "Filename", "Emulator", "Content"]:
        table.add_column(header=col, header_style="bold blue")
    for el in rows:
        table.add_row(
            str(el["id"]),
            get_success_text(str(el["enabled"])),
            get_datetime_text(el["created_at"]),
            el["filename"],
            el["emulator"],
            f"[cyan]{el['read']}[/]",
        )
    return table


def _generate_rule_table(rows: List[Dict]) -> Table:
    table = Table(
        show_header=True, title="Latest Rules", box=box.DOUBLE_EDGE, expand=True
    )
    for col in [
        "ID",
        "Enabled",
        "Name",
        "Meta",
        "Mitre\nTechnique",
        "Malware\nFamily",
        "Variables",
        "Weight",
    ]:
        table.add_column(header=col, header_style="bold blue")
    for el in rows:
        table.add_row(
            str(el["id"]),
            get_success_text(str(el["enabled"])),
            el["rule"],
            get_json_syntax(el["meta_description"]) if el["meta_description"] else None,
            f"[link={el['mitre_technique']['mitre_url']}]{el['mitre_technique']['name']}[/link]"
            if el["mitre_technique"]
            else None,
            el["malware_family"],
            get_json_syntax(el["variables"]) if el["variables"] else None,
            get_weight_text(el["weight"]),
        )
    return table


def _paginate_table(generator, generate_table_fn) -> None:
    console = Console()
    for resp, page in generator:
        console.print(generate_table_fn(resp.data["results"]), justify="center")
        console.print(
            f"page: {page}, total pages: {resp.data['total_pages']}",
            justify="center",
        )
        console.input("\n[ENTER] next page [CTRL+C] abort...")
        console.clear()



from unittest.mock import patch

from pydragonfly.sdk.const import ANALYZED, MALICIOUS
from tests.mock_utils import MockAPIResponse
from tests.resources import APIResourceBaseTestCase
from tests.resources.test_analysis import AnalysisResultTestCase


class DragonflyTestCase(APIResourceBaseTestCase):
    @property
    def resource(self):
        return self.df

    @patch(
        "pydragonfly.sdk.resources.analysis.Analysis.create",
        return_value=MockAPIResponse({"id": 1}, 200),
    )
    def test_analyze_file(self, *args, **kwargs):
        ret = self.df.analyze_file(
            sample_name="test", sample_buffer=b"test_sample", retrieve_analysis=False
        )
        self.assertEqual(ret, 1)

    @patch(
        "pydragonfly.sdk.resources.analysis.Analysis.retrieve",
        return_value=MockAPIResponse(AnalysisResultTestCase.result_json, 200),
    )
    @patch(
        "pydragonfly.sdk.resources.report.Report.matched_rules",
        return_value=MockAPIResponse(AnalysisResultTestCase.matched_rules_json, 200),
    )
    def test_analysis_result(self, *args, **kwargs):
        result = self.df.analysis_result(12)
        self.assertEqual(result.id, 12)
        self.assertEqual(result.status, ANALYZED)
        self.assertEqual(result.evaluation, MALICIOUS)
        self.assertEqual(result.score, 10)



from django_rest_client import (
    APIResource,
    SingletonAPIResourceMixin,
)


class UserAccessInfo(
    APIResource,
    SingletonAPIResourceMixin,
):
    """
    :class:`pydragonfly.Dragonfly.UserAccessInfo`
    """

    OBJECT_NAME = "api.me.access"
    EXPANDABLE_FIELDS = {
        "retrieve": [],
        "list": [],
    }
    ORDERING_FIELDS = []



from . import APIResourceBaseTestCase, APIResource

from tests.mock_utils import generic_204_mock


class InvitationResourceTestCase(APIResourceBaseTestCase):
    @property
    def resource(self) -> APIResource:
        return self.df.Invitation

    @generic_204_mock
    def test__accept(self, *args, **kwargs):
        response = self.resource.accept(object_id=self.object_id)
        self.assertEqual(204, response.code)

    @generic_204_mock
    def test__decline(self, *args, **kwargs):
        response = self.resource.decline(object_id=self.object_id)
        self.assertEqual(204, response.code)



from tests.mock_utils import generic_200_mock, generic_201_mock

from . import APIResource, APIResourceBaseTestCase


class RuleResourceTestCase(APIResourceBaseTestCase):
    @property
    def resource(self) -> APIResource:
        return self.df.Rule

    @generic_201_mock
    def test__create(self, *args, **kwargs):
        response = self.resource.create(
            data=self.resource.CreateRuleRequestBody(
                rule="testrule",
                weight=0,
                variables=["x", "y"],
                modules={
                    "and": [{"module": "Api", "syscall": "VirtualAlloc"}],
                    "order": True,
                },
                malware_family="test__create",
                mitre_technique="T1548",
                sensitive=False,
                meta_description={"author": "test__create"},
            )
        )
        self.assertEqual(201, response.code)

    @generic_200_mock
    def test__update(self, *args, **kwargs):
        response = self.resource.update(
            object_id=self.object_id,
            data=self.resource.UpdateRuleRequestBody(enabled=False),
        )
        self.assertEqual(200, response.code)

    @generic_200_mock
    def test__mitre(self, *args, **kwargs):
        response = self.resource.mitre()
        self.assertEqual(200, response.code)

    @generic_200_mock
    def test__aggregate_malware_behaviour(self, *args, **kwargs):
        response = self.resource.aggregate_malware_behaviour()
        self.assertEqual(200, response.code)

    @generic_200_mock
    def test__aggregate_mitre_technique(self, *args, **kwargs):
        response = self.resource.aggregate_mitre_technique()
        self.assertEqual(200, response.code)

    @generic_200_mock
    def test__aggregate_behaviour(self, *args, **kwargs):
        response = self.resource.aggregate_behaviour()
        self.assertEqual(200, response.code)



from . import APIResourceBaseTestCase, APIResource


from tests.mock_utils import generic_201_mock, generic_204_mock


class OrganizationResourceTestCase(APIResourceBaseTestCase):
    @property
    def resource(self) -> APIResource:
        return self.df.Organization

    @generic_201_mock
    def test__create(self, *args, **kwargs):
        response = self.resource.create(
            data=self.resource.CreateOrgRequestBody(name="test__create"),
        )
        self.assertEqual(201, response.code)

    @generic_201_mock
    def test__invite(self, *args, **kwargs):
        response = self.resource.invite(
            data=self.resource.InviteRequestBody(username="test")
        )
        self.assertEqual(201, response.code)

    @generic_204_mock
    def test__remove_member(self, *args, **kwargs):
        response = self.resource.remove_member(
            data=self.resource.InviteRequestBody(username="test")
        )
        self.assertEqual(204, response.code)



"""
# PyDragonfly
Robust Python SDK and CLI for interacting with Certego's Dragonfly service's API.
## Docs & Example Usage: https://github.com/certego/pydragonfly
"""

from pathlib import Path
from setuptools import setup, find_packages

# constants
GITHUB_URL = "https://github.com/certego/pydragonfly"

# The directory containing this file
HERE = Path(__file__).parent
# The text of the README file
README = (HERE / "README.md").read_text()
# Get requirements from files
requirements = (HERE / "requirements.txt").read_text().split("\n")
requirements_test = (HERE / "requirements.dev.txt").read_text().split("\n")
# read version
version_contents = {}
with open((HERE / "pydragonfly" / "version.py"), encoding="utf-8") as f:
    exec(f.read(), version_contents)

# This call to setup() does all the work
setup(
    name="pydragonfly",
    version=version_contents["VERSION"],
    description="Robust Python SDK and CLI for Certego's Dragonfly service's API",
    long_description=README,
    long_description_content_type="text/markdown",
    url=GITHUB_URL,
    author="Certego S.R.L",
    classifiers=[
        "Development Status :: 3 - Alpha",
        # Indicate who your project is intended for
        "Intended Audience :: Developers",
        # Pick your license as you wish (should match "license" above)
        "License :: OSI Approved :: BSD License",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
    packages=find_packages(),
    python_requires=">=3.6",
    include_package_data=True,
    install_requires=requirements,
    project_urls={
        "Documentation": GITHUB_URL,
        "Source": GITHUB_URL,
        "Tracker": f"{GITHUB_URL}/issues",
    },
    keywords="certego dragonfly sdk python command line osint threat intel malware",
    # List additional groups of dependencies here (e.g. development
    # dependencies). You can install these using the following syntax,
    # for example:
    # $ pip install -e .[dev,test]
    extras_require={
        "dev": requirements_test + requirements,
        "test": requirements_test + requirements,
    },
    # pip install --editable .
    entry_points="""
        [console_scripts]
        pydragonfly=pydragonfly.cli:cli
    """,
)



# system imports
from unittest import TestCase
from click.testing import CliRunner

# lib imports
from pydragonfly.cli import cli


class TestBasic(TestCase):
    def setUp(self):
        self.runner: CliRunner = CliRunner()

    def test__cli_invoke(self):
        result = self.runner.invoke(cli)
        self.assertEqual(result.exit_code, 0)
        self.assertIn("Usage", result.output)



import logging
import time
from typing import List, Union

from django_rest_client import APIClient
from django_rest_client.types import THeaders, Toid

from ..version import VERSION
from .resources import (
    Action,
    Analysis,
    Invitation,
    Organization,
    Profile,
    Report,
    Rule,
    Sample,
    Session,
    UserAccessInfo,
    UserPreferences,
)
from .resources.analysis import AnalysisResult


class Dragonfly(APIClient):
    # overwrite
    _server_url: str = "https://dragonfly.certego.net"

    @property
    def _headers(self) -> THeaders:
        return {
            **super()._headers,
            "User-Agent": f"PyDragonfly/{VERSION}",
        }

    def __init__(self, api_key: str, logger: logging.Logger = None):
        super().__init__(api_key, None, logger)

    # resources
    Action = Action
    Analysis = Analysis
    Invitation = Invitation
    Organization = Organization
    Profile = Profile
    Report = Report
    Rule = Rule
    Sample = Sample
    Session = Session
    UserAccessInfo = UserAccessInfo
    UserPreferences = UserPreferences

    # utilities

    def analyze_file(
        self,
        sample_name: str,
        sample_buffer: bytes,
        retrieve_analysis: bool = True,
        profiles: List[int] = None,
        private: bool = False,
        root: bool = False,
        operating_system: str = None,
        arguments: List[str] = None,
        dll_entrypoints: List[str] = None,
    ) -> Union[AnalysisResult, int]:
        """
        Utility function to create a new analysis and get analysis ID
        or optionally receive result directly.

        Args:
            ``sample_name`` (str):
             Name of the sample to analyze.\n
            ``sample_buffer`` (bytes):
             Sample buffer in bytes form.\n
            ``retrieve_analysis`` (bool, optional):
             If ``True``, fetch and return result otherwise return only analysis ID.
             Default ``True``.\n
            ``profiles`` (List[int], optional):
             List of IDs of profiles to emulate against.
             Default ``[1,2]``.\n
            ``private`` (bool, optional):
             Mark analysis as private limitting access to you
             and members in your organization only. Requires paid subscription.
             Default ``False``.\n
            ``root`` (bool, optional):
             Emulate with root permissions on OS level.
             Default ``False``.\n
            ``operating_system`` (str, optional):
             OS of the given sample. Default ``None`` i.e. detected by dragonfly.\n
            ``arguments`` (List[str], optional):
             List of extra CLI arguments to pass to the emulator.
             Only use if you know what you are doing.
             Default ``None``.\n
            ``dll_entrypoints`` (List[str], optional):
             DLL entrypoints.
             Default ``None``.

        .. versionadded:: 0.0.4
        """
        if profiles is None:
            profiles = [1, 2]
        data = self.Analysis.CreateAnalysisRequestBody(
            # We have 2 defaults profile, one for qiling, one for speakeasy.
            profiles=profiles,
            # right now we do not support private analysis anyway
            private=private,
            # emulation hooks on rule matching. Not released yet
            allow_actions=False,
            # your wish here, imho executing thing as users is more common
            root=root,
            # we can detected the OS on our backend.
            # It is required if you want to analyze shellcodes unfortunately
            os=operating_system,
            # the safer approach is that the sample did not require specific arguments
            arguments=arguments,
            # if not entrypoints are selected, and the sample is a dll
            # we will emulate a maximum of 100 entrypoints
            dll_entrypoints=dll_entrypoints,
        )
        resp = self.Analysis.create(
            data=data, sample_name=sample_name, sample_buffer=sample_buffer
        ).data
        analysis_id = resp["id"]
        if retrieve_analysis:
            return self.analysis_result(analysis_id)
        else:
            return analysis_id

    def analysis_result(
        self,
        analysis_id: Toid,
        waiting_time: int = 10,
        max_wait_cycle: int = 30,  # 30 x 10 = 5 mins
    ) -> AnalysisResult:
        """
        Utility function to retrieve an analysis' result.

        Total waiting time = ``waiting_time x max_wait_cycle``.

        Args:
           ``analysis_id`` (int|str):
            Analysis ID to fetch result of.\n
           ``waiting_time`` (int, optional):
            Wait time between subsequent HTTP requests. Default ``10``.\n
           ``max_wait_cycle`` (int, optional):
            Maximum number of HTTP requests. Default ``30``.

        .. versionadded:: 0.0.4
        """
        result = self.Analysis.Result(analysis_id)
        if max_wait_cycle:
            waiting_cycle: int = 0
            while not result.is_ready() and waiting_cycle < max_wait_cycle:
                time.sleep(waiting_time)
                result.refresh()
                waiting_cycle += 1
        return result
